/**********************************************************************************/
/*  vproweather 1.0.9                                                            */
/*      A program for acquiring data from a Davis Vantage Pro Weather Station.    */
/*                                                                                */
/*  Thanks to Paul Davis for his 'remserial' program, and Aaron Logue's           */
/*  'weastat' program.                                                            */
/*                                                                                */
/* (c)2004 Joe Jaworski    email: jj@joejaworski.com                              */
/* (c)2005-2013 Curt Blank email: curt@curtronics.com                             */
/* VPROWEATHER is covered under the GNU general license. See the attached         */
/* LICENSE file for details.                                                      */
/*                                                                                */
/* This software is provided as-is, without any expressed or implied              */
/* warranties, including, without limitation, the implied warranties of           */
/* merchantibility and fitness for any particular purpose.                        */
/*                                                                                */
/*     v0.6-9  cjb - modified graphic data sets-memory pointer offsets due        */
/*                   change in locations of the graph data in Vantage Pro2        */
/*     v0.8.0  cjb - add access to mySQL db                                       */
/*     v0.8-10 cjb - improve Barometer Trend documentation                        */
/*     v0.8-41 cjb - use __attribute__ ((packed)) on structures because can no    */
/*                   compile using -fpack-struct                                  */
/*     v0.9.0  cjb - added data for handling DMP & DMPAFT                         */
/*     v0.9.6  cjb - pass bRTDSuccess to OutputData                               */
/*     v1.0.4  cjb - added -2 option for use with Vantage Pro 2 and removed       */
/*                   compile flag, now a runtime option                           */
/*     v1.0.7  cjb - added bTipMod                                                */
/*                                                                                */
/* dhandler.h - data handlers include file                                        */
/*                                                                                */
/* Vantage Pro uses Big-Endian convention                                         */
/*                                                                                */
/**********************************************************************************/

#ifndef __DHANDLER_H__
#define __DHANDLER_H__

#include "main.h"

#ifdef __cplusplus
#define extern "C" {            /* respect c++ callers */
#endif

/* need these to calculate sunrise, sunset, and solar noon  */
/* remove "//" from beginning of line when you define these */
#define LATITUDE      -1000.0
#define LONGITUDE     -1000.0

#define _OVERFLOW_MSG	"overflow error"

/* exports */
extern void setupVP1();
extern void setupVP2();
extern int CheckCRC(int nCnt, char *pData);
extern time_t DateTimeCT(char * s1);
extern void GetRTData(char *szData);
extern void GetDMPData(char *szData);
extern void DumpData(char *szData, int gCnt, char loopFN[]);
extern int GetData(char *szData, int cCnt, char loopFN[]);
extern void GetHLData(char *szData);
extern void PrintRTData(void);
extern void PrintRTDataFmt(char *fString);
extern void PrintHLData(void);
extern void PrintGDData(char * pData);
extern void PrintDMPData(int seq, int page, int rec, char mark[]);
extern char* getCurTime(void);
extern void PrintTime(char *szData);
extern void ListOutputDataCols(char outFN[]);
extern void OutputData(char * pData, char outFN[], BOOL bRTDSuccess, BOOL bGDSuccess);

extern char* TimeConvert(WORD wTime);
extern char* BaroTrend(int ns);
extern char* BaroTrendImage();
extern char* WindRoseDir();
extern char* PrintDate(WORD wDate);
extern char* PrintDateDMP(WORD wDate);
extern float calcWindChill(float Temperature, float WindSpeed);
extern float calcHeatIndex(float Temperature, float RelHumidity);
extern char* SubString(const char* text, char delim, int *ppos);
extern int SetEnvVar(char * pData, char envVars[]);
extern void calcSunRiseSetNoon(float lat, float lng, int year, int month, int day, int hour, int minute, int second, int dst, float tz);

extern char* TimeRef10Min(time_t tt);
extern char* TimeRef15Min(time_t tt);
extern char* TimeRefHours(time_t tt);
extern char* TimeRefDays(time_t tt);
extern char* TimeRefMonths(time_t tt);
extern char* TimeRefYears(time_t tt);

extern char* PrintTempSet24(BYTE *pData, int ptrOffset, BYTE rrOffset, int valOffset);
extern char* PrintByteSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize, int cvtType);
extern char* PrintSolarSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);
extern char* PrintBarSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);
extern char* PrintETByteSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);
extern char* PrintRainRateSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);
extern char* PrintRainRateByteSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);
extern char* PrintDateSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);

/* imports */
extern BOOL bVerbose;                   /* Verbose Mode */
extern BOOL bVpro2;
extern BOOL bMtime;
extern BOOL bZcommas;
extern BOOL bCalcSun;
extern BOOL bSolarNoonSec;
extern BOOL bSwapEndian;
extern BOOL bMetric;
extern BOOL bTipMod;
extern time_t ttRun;
extern int dstRun;
extern char rtDateTimeCT[20];
extern char hlDateTimeCT[20];
extern char grDateTimeCT[20];
extern time_t ttrt;
extern time_t tthl;
extern time_t ttgr;
extern char SunRise[26];
extern char SunSet[26];
extern char SolarNoon[11];

extern BOOL toSTDOUT(BOOL success, BOOL format);


#define HeaderStrLen    270

/* VPro chars */
#define VP_ACK          0x06
#define VP_NAK          0x21
#define VP_LF           0x0a
#define VP_CR           0x0d
#define VP_SP           0x20
#define VP_CANCEL       0x18
#define VP_ESC          0x1b
#define VP_LAT          0x0b
#define VP_LON          0x0d

#define _CVT_NONE	0
#define _CVT_TEMP	1
#define _CVT_WIND	2
#define _CVT_RAIN	3
#define _CVT_BARO	4
#define _CVT_ET_OZ	5
#define _CVT_ET_QT	6
#define _CVT_ET_GL	7

/* graphic data sets- memory pointer offsets */
#define EE_LATITUDE             12
#define EE_LONGITUDE            14

#define RAIN_OZINCH             79.7913682277

/* Definition of Davis LOOP data packet */
typedef struct t_RTDATA
{
        BYTE    yACK;           /* -1 ACK from stream                                                   */
        char    cL;             /* 0  character "L"                                                     */
        char    cO;             /* 1  character "O"                                                     */
        char    cO1;            /* 2  character "O"                                                     */
        char    cP;             /* 3  character "P" (RevA) or the current                               */
                                /*    3-hour Barometer trend as follows:                                */
                                /*    -60 = Falling Rapidly = 196 (as an unsigned byte)                 */
                                /*    -20 = Falling Slowly = 236 (as an unsigned byte)                  */
                                /*    0   = Steady                                                      */
                                /*    20  = Rising Slowly                                               */
                                /*    60  = Rising Rapidly                                              */
                                /*    80 = ASCII "P" = Rev A firmware, no trend info is available       */
                                /* any other value is 3-hour data not available */
        BYTE    yPacketType;    /* 4 Always zero for current firmware release                           */
        WORD    wNextRec;       /* 5 loc in archive memory for next data packet                         */
        WORD    wBarometer;     /* 7 Current barometer as (Hg / 1000)                                   */
        WORD    wInsideTemp;    /* 9 Inside Temperature as (DegF / 10)                                  */
        BYTE    yInsideHum;     /* 11 Inside Humidity as percentage                                     */
        WORD    wOutsideTemp;   /* 12 Outside Temperature as (DegF / 10)                                */
        BYTE    yWindSpeed;     /* 14 Wind Speed                                                        */
        BYTE    yAvgWindSpeed;  /* 15 10-Minute Average Wind Speed                                      */
        WORD    wWindDir;       /* 16 Wind Direction in degress                                         */
        BYTE    yXtraTemps[7];  /* 18 Extra Temperatures                                                */
        BYTE    ySoilTemps[4];  /* 25 Soil Temperatures                                                 */
        BYTE    yLeafTemps[4];  /* 29 Leaf Temperatures                                                 */
        BYTE    yOutsideHum;    /* 33 Outside Humidity                                                  */
        BYTE    yXtraHums[7];   /* 34 Extra Humidities                                                  */
        WORD    wRainRate;      /* 41 Rain Rate                                                         */
        BYTE    yUVLevel;       /* 43 UV Level                                                          */
        WORD    wSolarRad;      /* 44 Solar Radiation                                                   */
        WORD    wStormRain;     /* 46 Total Storm Rain                                                  */
        WORD    wStormStart;    /* 48 Start date of current storm                                       */
        WORD    wRainDay;       /* 50 Rain Today                                                        */
        WORD    wRainMonth;     /* 52 Rain this Month                                                   */
        WORD    wRainYear;      /* 54 Rain this Year                                                    */
        WORD    wETDay;         /* 56 Day ET                                                            */
        WORD    wETMonth;       /* 58 Month ET                                                          */
        WORD    wETYear;        /* 60 Year ET                                                           */
        BYTE    wSoilMoist[4];  /* 62 Soil Moistures                                                    */
        BYTE    wLeafWet[4];    /* 66 Leaf Wetness                                                      */
        BYTE    yAlarmInside;   /* 70 Inside Alarm bits                                                 */
        BYTE    yAlarmRain;     /* 71 Rain Alarm bits                                                   */
        WORD    yAlarmOut;      /* 72 Outside Temperature Alarm bits                                    */
        BYTE    yAlarmXtra[8];  /* 74 Extra Temp/Hum Alarms                                             */
        BYTE    yAlarmSL[4];    /* 82 Soil and Leaf Alarms                                              */
        BYTE    yXmitBatt;      /* 86 Transmitter battery status                                        */
        WORD    wBattLevel;     /* 87 Console Battery Level:                                            */
                                        /*    Voltage = ((wBattLevel * 300)/512)/100.0                  */
        BYTE    yForeIcon;      /* 89 Forecast Icon                                                     */
        BYTE    yRule;          /* 90 Forecast rule number                                              */
        WORD    wSunrise;       /* 91 Sunrise time (BCD encoded, 24hr)                                  */
        WORD    wSunset;        /* 93 Sunset time  (BCD encoded, 24hr)                                  */
        BYTE    yLF;            /* 95 Line Feed (\n) 0x0a                                               */
        BYTE    yCR;            /* 96 Carraige Return (\r) 0x0d                                         */
        WORD    WCRC;           /* 97 CRC check bytes (CCITT-16 standard)                               */
} __attribute__ ((packed)) RTDATA;

/* Definition of Davis HILOW packet */
typedef struct t_HLDATA
{
        BYTE    yACK;           /* -1 ACK from stream                                   */

        /* barometer */
        WORD    wBaroLoDay;     /* 0   0x00 Low barometer for today                             */
        WORD    wBaroHiDay;     /* 2   0x02 High barometer for today                            */
        WORD    wBaroLoMonth;   /* 4   0x04 Low barometer this month                            */
        WORD    wBaroHiMonth;   /* 6   0x06 High barometer this month                           */
        WORD    wBaroLoYear;    /* 8   0x08 Low barometer this year                             */
        WORD    wBaroHiYear;    /* 10  0x0a High barometer this year                            */
        WORD    wBaroLoTime;    /* 12  0x0c Low barometer time of day                           */
        WORD    wBaroHiTime;    /* 14  0x0e High barometer time of day                  */

        /* wind speed */
        BYTE    yWindHiDay;     /* 16  0x10 Highest wind speed for today                        */
        WORD    wWindHiTime;    /* 17  0x11 Highest wind speed time of day                      */
        BYTE    yWindHiMonth;   /* 19  0x13 Highest wind speed for the month                    */
        BYTE    yWindHiYear;    /* 20  0x14 Highest wind speed for the year                     */

        /* inside temperatures */
        WORD    wInTempHiDay;   /* 21  0x15 Inside high temp for today                  */
        WORD    wInTempLoDay;   /* 23  0x17 Inside low temp for today                           */
        WORD    wInTempHiTime;  /* 25  0x19 Time of Inside high temp for today          */
        WORD    wInTempLoTime;  /* 27  0x1b Time of Inside low temp for today                   */
        WORD    wInTempLoMonth; /* 29  0x1d Inside low temp for the month                       */
        WORD    wInTempHiMonth; /* 31  0x1f Inside high temp for the month                      */
        WORD    wInTempLoYear;  /* 33  0x21 Inside low temp for the year                        */
        WORD    wInTempHiYear;  /* 35  0x23 Inside high temp for the year                       */

        /* Inside Humidity */
        BYTE    yInHumHiDay;    /* 37  0x25 Inside high humidity for the day                    */
        BYTE    yInHumLoDay;    /* 38  0x26 Inside low humidity for the day                     */
        WORD    wInHumHiTime;   /* 39  0x27 Inside high humidity time for today         */
        WORD    wInHumLoTime;   /* 41  0x29 Inside low humidity time for today          */
        BYTE    yInHumHiMonth;  /* 43  0x2b Inside high humidity for the month          */
        BYTE    yInHumLoMonth;  /* 44  0x2c Inside low humidity for the month                   */
        BYTE    yInHumHiYear;   /* 45  0x2d Inside high humidity for the year                   */
        BYTE    yInHumLoYear;   /* 46  0x2e Inside low humidity for the year                    */

        /* outside temperatures */
        WORD    wTempLoDay;     /* 47  0x2f Outside low temp for today                  */
        WORD    wTempHiDay;     /* 49  0x31 Outside high temp for today                 */
        WORD    wTempLoTime;    /* 51  0x33 Time of Outside low temp for today          */
        WORD    wTempHiTime;    /* 53  0x35 Time of Outside high temp for today         */
        WORD    wTempHiMonth;   /* 55  0x37 Outside high temp for the month                     */
        WORD    wTempLoMonth;   /* 57  0x39 Outside low temp for the month                      */
        WORD    wTempHiYear;    /* 59  0x3b Outside high temp for the year                      */
        WORD    wTempLoYear;    /* 61  0x3d Outside low temp for the year                       */

        /* dew point */
        WORD    wDewLoDay;      /* 63  0x3f dew point low for today                             */
        WORD    wDewHiDay;      /* 65  0x41 dew point high for today                            */
        WORD    wDewLoTime;     /* 67  0x43 Time of dew point low for today                     */
        WORD    wDewHiTime;     /* 69  0x45 Time of dew point high for today                    */
        WORD    wDewHiMonth;    /* 71  0x47 Highest dew point this month                        */
        WORD    wDewLoMonth;    /* 73  0x49 Lowest dew point this month                 */
        WORD    wDewHiYear;     /* 75  0x4b Highest dew point for the year                      */
        WORD    wDewLoYear;     /* 77  0x4d Lowest dew point for the year                       */

        /* wind chill */
        WORD    wChillLoDay;    /* 79  0x4f wind chill low for today                            */
        WORD    wChillLoTime;   /* 81  0x51 Time of wind chill low for today                    */
        WORD    wChillLoMonth;  /* 83  0x53 Lowest wind chill this month                        */
        WORD    wChillLoYear;   /* 85  0x55 Lowest wind chill for the year                      */

        /* heat indices */
        WORD    wHeatHiDay;     /* 87  0x57 Heat index high for today                           */
        WORD    wHeatHiTime;    /* 89  0x59 Time of heat index high for today                   */
        WORD    wHeatHiMonth;   /* 91  0x5b Heat index high for the month                       */
        WORD    wHeatHiYear;    /* 93  0x5d Heat index high for the year                        */

        /* THSW indices */
        WORD    wTHSWHiDay;     /* 95  0x5f THSW index high for today                           */
        WORD    wTHSWHiTime;    /* 97  0x61 Time of THSW index high for today                   */
        WORD    wTHSWHiMonth;   /* 99  0x63 THSW index high for the month                       */
        WORD    wTHSWHiYear;    /* 101 0x65 THSW index high for the year                        */

        /* Solar Radiation */
        WORD    wSolarHiDay;    /* 103 0x67 Solar rad high for today                            */
        WORD    wSolarHiTime;   /* 105 0x69 Time of Solar rad high for today                    */
        WORD    wSolarHiMonth;  /* 107 0x6b Solar rad high for the month                        */
        WORD    wSolarHiYear;   /* 109 0x6d Solar rad high for the year                 */

        /* UV Index */
        BYTE    yUVHiDay;       /* 111 0x6f UV high for today                           */
        WORD    wUVHiTime;      /* 112 0x70 Time of UV high for today                   */
        BYTE    yUVHiMonth;     /* 114 0x72 UV high for the month                               */
        BYTE    yUVHiYear;      /* 115 0x73 UV high for the year                                */

        /* Rain Rate */
        WORD    wRainHiDay;     /* 116 0x74 Rain Rate high for today                            */
        WORD    wRainHiTime;    /* 118 0x76 Time of Rain Rate high for today                    */
        WORD    wRainHiHour;    /* 120 0x78 Highest Rain Rate this hour                 */
        WORD    wRainHiMonth;   /* 122 0x7a Highest Rain Rate this month                        */
        WORD    wRainHiYear;    /* 124 0x7c Highest Rain Rate this year                 */

        BYTE    yExtraLeaf[150];/* 126 0x Extra/Leaf/Soil Temperatures                  */
        BYTE    yExtraTemps[80];/* 276 0x Extra outside temp/Humidities                 */
        BYTE    ySoilMoist[40]; /* 356 0x Soil Moisture section                         */
        BYTE    yLeafWet[40];   /* 396 0x Leaf Wetness section                          */
        WORD    wUnknown;       /* 436 not documented                                   */
        WORD    WCRC;           /* 438 CRC check bytes (CCITT-16 standard)              */
} __attribute__ ((packed)) HLDATA;

typedef struct t_DMPDATA_A
{
        WORD    wDate;                  /* 0 The date that the archive was written. */
        WORD    wTime;                  /* 2 ime on the Vantage that the archive record was written. */
        WORD    wTempOut;               /* 4 Either the Average Outside Temperature, or the Final Outside Temperature. Units are (°F / 10) */
        WORD    wTempOutHi;             /* 6 Highest Outside Temp over the archive period. */
        WORD    wTempOutLo;             /* 8 Lowest Outside Temp over the archive period. */
        WORD    wRainFall;              /* 10 Number of rain clicks over the archive period */
        WORD    wRainRateHi;            /* 12 Highest rain rate over the archive period, Units are (rain clicks / hour) */
        WORD    wBarometer;             /* 14 Barometer reading at the end of the archive period. Units are (in Hg / 1000) */
        WORD    wSolarRad;              /* 16 Average Solar Rad over the archive period. Units are (Watts / m2) */
        WORD    wWindSamples;           /* 18 Number of packets containing wind speed data */
        WORD    wTempIn;                /* 20 Either the Average Inside Temperature, or the Final Inside Temperature (°F / 10) */
        BYTE    yHumIn;                 /* 22 Inside Humidity at the end of the archive period */
        BYTE    yHumOut;                /* 23 Outside Humidity at the end of the archive period */
        BYTE    yWindSpeedAvg;          /* 24 Average Wind Speed over the archive interval. Unit are (MPH) */
        BYTE    yWindSpeedHi;           /* 25 Highest Wind Speed over the archive interval. Units are (MPH) */
        BYTE    yWindSpeedDirHi;        /* 26 Direction code of the High Wind speed. */
        BYTE    yWindDirPrevail;        /* 27 Prevailing or Dominant Wind Direction code. */
        BYTE    yUVAvg;                 /* 28 Average UV Index. Units are (UV Index / 10) */
        BYTE    yET;                    /* 29 ET accumulated over the last hour. Units are (in /1000) */
        BYTE    yInvalid;               /* 30 */
        BYTE    ySoilMoist1;            /* 31 Soil Moisture values. Units are (cb) */
        BYTE    ySoilMoist2;            /* 32 Soil Moisture values. Units are (cb) */
        BYTE    ySoilMoist3;            /* 33 Soil Moisture values. Units are (cb) */
        BYTE    ySoilMoist4;            /* 34 Soil Moisture values. Units are (cb) */
        BYTE    ySoilTemp1;             /* 35 Soil Temperatures. Units are (°F + 90) */
        BYTE    ySoilTemp2;             /* 36 Soil Temperatures. Units are (°F + 90) */
        BYTE    ySoilTemp3;             /* 37 Soil Temperatures. Units are (°F + 90) */
        BYTE    ySoilTemp4;             /* 38 Soil Temperatures. Units are (°F + 90) */
        BYTE    yLeafWet1;              /* 39 Leaf Wetness values. Range is 0 – 15 */
        BYTE    yLeafWet2;              /* 40 Leaf Wetness values. Range is 0 – 15 */
        BYTE    yLeafWet3;              /* 41 Leaf Wetness values. Range is 0 – 15 */
        BYTE    yLeafWet4;              /* 42 Leaf Wetness values. Range is 0 – 15 */
        BYTE    yExtraTemp1;            /* 43 Extra Temperature values. Units are (°F + 90) */
        BYTE    yExtraTemp2;            /* 44 Extra Temperature values. Units are (°F + 90) */
        BYTE    yExtraHum1;             /* 45 Extra Humidity values */
        BYTE    yExtraHum2;             /* 46 Extra Humidity values */
        WORD    wReedClosed;            /* 47 Count of the number of time the anemometer reed switch was closed */
        WORD    wReedOpened;            /* 49 Count of the number of time the anemometer reed switch was opened */
        BYTE    yUnused;                /* 51 */
} __attribute__ ((packed)) DMPDATA_A;

typedef struct t_DMPDATA_B
{
        WORD    wDate;                  /* 0 The date that the archive was written. */
        WORD    wTime;                  /* 2 ime on the Vantage that the archive record was written. */
        WORD    wTempOut;               /* 4 Either the Average Outside Temperature, or the Final Outside Temperature. Units are (°F / 10) */
        WORD    wTempOutHi;             /* 6 Highest Outside Temp over the archive period. */
        WORD    wTempOutLo;             /* 8 Lowest Outside Temp over the archive period. */
        WORD    wRainFall;              /* 10 Number of rain clicks over the archive period */
        WORD    wRainRateHi;            /* 12 Highest rain rate over the archive period, Units are (rain clicks / hour) */
        WORD    wBarometer;             /* 14 Barometer reading at the end of the archive period. Units are (in Hg / 1000) */
        WORD    wSolarRad;              /* 16 Average Solar Rad over the archive period. Units are (Watts / m2) */
        WORD    wWindSamples;           /* 18 Number of packets containing wind speed data */
        WORD    wTempIn;                /* 20 Either the Average Inside Temperature, or the Final Inside Temperature (°F / 10) */
        BYTE    yHumIn;                 /* 22 Inside Humidity at the end of the archive period */
        BYTE    yHumOut;                /* 23 Outside Humidity at the end of the archive period */
        BYTE    yWindSpeedAvg;          /* 24 Average Wind Speed over the archive interval. Unit are (MPH) */
        BYTE    yWindSpeedHi;           /* 25 Highest Wind Speed over the archive interval. Units are (MPH) */
        BYTE    yWindSpeedDirHi;        /* 26 Direction code of the High Wind speed. */
        BYTE    yWindDirPrevail;        /* 27 Prevailing or Dominant Wind Direction code. */
        BYTE    yUVAvg;                 /* 28 Average UV Index. Units are (UV Index / 10) */
        BYTE    yET;                    /* 29 ET accumulated over the last hour. Units are (in /1000) */
        WORD    wSolarRadHi;            /* 30 Highest Solar Rad value over the archive period. Units are (Watts / m2) */
        BYTE    yUVIndexHi;             /* 32 Highest UV Index value over the archive period. Units are (Watts / m2) */
        BYTE    yForecastRule;          /* 33 Weather forecast rule at the end of the archive period. */
        BYTE    yLeafTemp1;             /* 34 Leaf Temperature values. Units are (°F + 90) */
        BYTE    yLeafTemp2;             /* 35 Leaf Temperature values. Units are (°F + 90) */
        BYTE    yLeafWet1;              /* 36 Leaf Wetness values. Range is 0 – 15 */
        BYTE    yLeafWet2;              /* 37 Leaf Wetness values. Range is 0 – 15 */
        BYTE    ySoilTemp1;             /* 38 Soil Temperatures. Units are (°F + 90) */
        BYTE    ySoilTemp2;             /* 39 Soil Temperatures. Units are (°F + 90) */
        BYTE    ySoilTemp3;             /* 40 Soil Temperatures. Units are (°F + 90) */
        BYTE    ySoilTemp4;             /* 41 Soil Temperatures. Units are (°F + 90) */
        BYTE    yRecType;               /* 42 0xFF = Rev A, 0x00 = Rev B archive record */
        BYTE    yExtraHum1;             /* 43 Extra Humidity values */
        BYTE    yExtraHum2;              /* 44 Extra Humidity values */
        BYTE    yExtraTemp1;            /* 45 Extra Temperature values. Units are (°F + 90) */
        BYTE    yExtraTemp2;            /* 46 Extra Temperature values. Units are (°F + 90) */
        BYTE    yExtraTemp3;            /* 47 Extra Temperature values. Units are (°F + 90) */
        BYTE    ySoilMoist1;            /* 48 Soil Moisture values. Units are (cb) */
        BYTE    ySoilMoist2;            /* 49 Soil Moisture values. Units are (cb) */
        BYTE    ySoilMoist3;            /* 50 Soil Moisture values. Units are (cb) */
        BYTE    ySoilMoist4;            /* 51 Soil Moisture values. Units are (cb) */
} __attribute__ ((packed)) DMPDATA_B;


/* Definition of Davis EEPROM data packet */
typedef struct t_EEDATA
{
        BYTE    szDumm[177];    /* stuff not used in this version               */

} __attribute__ ((packed)) EEDATA;


#define GRAPH_START              177
    
#define NEXT_10MIN_PTR           GRAPH_START+1
#define NEXT_15MIN_PTR           GRAPH_START+2
#define NEXT_HOUR_PTR            GRAPH_START+3
#define NEXT_DAY_PTR             GRAPH_START+4
#define NEXT_MONTH_PTR           GRAPH_START+5
#define NEXT_YEAR_PTR            GRAPH_START+6
#define NEXT_RAIN_STORM_PTR      GRAPH_START+7
#define NEXT_RAIN_YEAR_PTR       GRAPH_START+8

extern int BAR_15_MIN;
extern int BAR_DAY_HIGHS;
extern int BAR_DAY_HIGH_TIMES;
extern int BAR_DAY_LOWS;
extern int BAR_DAY_LOW_TIMES;
extern int BAR_HOUR;
extern int BAR_MONTH_HIGHS;
extern int BAR_MONTH_LOWS;
extern int BAR_YEAR_HIGHS;
extern int BAR_YEAR_LOWS;
extern int BAUD_RATE;
extern int CHILL_DAY_LOWS;
extern int CHILL_DAY_LOW_TIMES;
extern int CHILL_HOUR;
extern int CHILL_MONTH_LOWS;
extern int CHILL_YEAR_LOWS;
extern int DEFAULT_RATE_GRAPH;
extern int DEW_DAY_HIGHS;
extern int DEW_DAY_HIGH_TIMES;
extern int DEW_DAY_LOWS;
extern int DEW_DAY_LOW_TIMES;
extern int DEW_HOUR;
extern int DEW_MONTH_HIGHS;
extern int DEW_MONTH_LOWS;
extern int DEW_YEAR_HIGHS;
extern int DEW_YEAR_LOWS;
extern int ET_DAY_TOTAL;
extern int ET_HOUR;
extern int ET_MONTH_TOTAL;
extern int ET_YEAR_TOTAL;
extern int HEAT_DAY_HIGHS;
extern int HEAT_DAY_HIGH_TIMES;
extern int HEAT_HOUR;
extern int HEAT_MONTH_HIGHS;
extern int HEAT_YEAR_HIGHS;
extern int HUM_IN_DAY_HIGHS;
extern int HUM_IN_DAY_HIGH_TIMES;
extern int HUM_IN_DAY_LOWS;
extern int HUM_IN_DAY_LOW_TIMES;
extern int HUM_IN_HOUR;
extern int HUM_IN_MONTH_HIGHS;
extern int HUM_IN_MONTH_LOWS;
extern int HUM_IN_YEAR_HIGHS;
extern int HUM_IN_YEAR_LOWS;
extern int HUM_OUT_DAY_HIGHS;
extern int HUM_OUT_DAY_HIGH_TIMES;
extern int HUM_OUT_DAY_LOWS;
extern int HUM_OUT_DAY_LOW_TIMES;
extern int HUM_OUT_HOUR;
extern int HUM_OUT_MONTH_HIGHS;
extern int HUM_OUT_MONTH_LOWS;
extern int HUM_OUT_YEAR_HIGHS;
extern int HUM_OUT_YEAR_LOWS;
extern int LEAF_DAY_HIGHS;
extern int LEAF_DAY_HIGH_TIMES;
extern int LEAF_DAY_LOWS;
extern int LEAF_DAY_LOW_TIMES;
extern int LEAF_HOUR;
extern int LEAF_MONTH_HIGHS;
extern int LEAF_MONTH_LOWS;
extern int LEAF_YEAR_HIGHS;
extern int LEAF_YEAR_LOWS;
extern int RAIN_15_MIN;
extern int RAIN_DAY_TOTAL;
extern int RAIN_HOUR;
extern int RAIN_MONTH_TOTAL;
extern int RAIN_RATE_1_MIN;
extern int RAIN_RATE_DAY_HIGHS;
extern int RAIN_RATE_DAY_HIGH_TIMES;
extern int RAIN_RATE_HOUR;
extern int RAIN_RATE_MONTH_HIGHS;
extern int RAIN_RATE_YEAR_HIGHS;
extern int RAIN_STORM;
extern int RAIN_STORM_END;
extern int RAIN_STORM_START;
extern int RAIN_YEAR_TOTAL;
extern int RX_PERCENTAGE;
extern int SAVE_DAY;
extern int SAVE_HOUR;
extern int SAVE_MIN;
extern int SAVE_MONTH;
extern int SAVE_YEAR;
extern int SAVE_YEAR_COMP;
extern int SOIL_DAY_HIGHS;
extern int SOIL_DAY_HIGH_TIMES;
extern int SOIL_DAY_LOWS;
extern int SOIL_DAY_LOW_TIMES;
extern int SOIL_HOUR;
extern int SOIL_MONTH_HIGHS;
extern int SOIL_MONTH_LOWS;
extern int SOIL_YEAR_HIGHS;
extern int SOIL_YEAR_HIGHS_COMP;
extern int SOIL_YEAR_LOWS;
extern int SOLAR_DAY_HIGHS;
extern int SOLAR_DAY_HIGH_TIMES;
extern int SOLAR_HOUR_AVG;
extern int SOLAR_MONTH_HIGHS;
extern int SOLAR_YEAR_HIGHS;
extern int TEMP_IN_DAY_HIGHS;
extern int TEMP_IN_DAY_HIGH_TIMES;
extern int TEMP_IN_DAY_LOWS;
extern int TEMP_IN_DAY_LOW_TIMES;
extern int TEMP_IN_HOUR;
extern int TEMP_IN_MONTH_HIGHS;
extern int TEMP_IN_MONTH_LOWS;
extern int TEMP_IN_YEAR_HIGHS;
extern int TEMP_IN_YEAR_LOWS;
extern int TEMP_OUT_DAY_HIGHS;
extern int TEMP_OUT_DAY_HIGH_TIMES;
extern int TEMP_OUT_DAY_LOWS;
extern int TEMP_OUT_DAY_LOW_TIMES;
extern int TEMP_OUT_HOUR;
extern int TEMP_OUT_MONTH_HIGHS;
extern int TEMP_OUT_MONTH_LOWS;
extern int TEMP_OUT_YEAR_HIGHS;
extern int TEMP_OUT_YEAR_LOWS;
extern int THSW_DAY_HIGHS;
extern int THSW_DAY_HIGH_TIMES;
extern int THSW_HOUR;
extern int THSW_MONTH_HIGHS;
extern int THSW_YEAR_HIGHS;
extern int UV_DAY_HIGHS;
extern int UV_DAY_HIGH_TIMES;
extern int UV_HOUR_AVG;
extern int UV_MEDS_DAY;
extern int UV_MEDS_HOUR;
extern int UV_MONTH_HIGHS;
extern int UV_YEAR_HIGHS;
extern int WIND_DIR_DAY;
extern int WIND_DIR_DAY_BINS;
extern int WIND_DIR_HOUR;
extern int WIND_DIR_MONTH;
extern int WIND_DIR_MONTH_BINS;
extern int WIND_SPEED_10_MIN_AVG;
extern int WIND_SPEED_DAY_HIGHS;
extern int WIND_SPEED_DAY_HIGH_DIR;
extern int WIND_SPEED_DAY_HIGH_TIMES;
extern int WIND_SPEED_HOUR_AVG;
extern int WIND_SPEED_HOUR_HIGHS;
extern int WIND_SPEED_MONTH_HIGHS;
extern int WIND_SPEED_MONTH_HIGH_DIR;
extern int WIND_SPEED_YEAR_HIGHS;
extern int WIND_SPEED_YEAR_HIGH_DIR;

#ifdef __cplusplus
}
#endif /* __cplusplus    */
#endif /* __DHANDLER_H__ */
